import type { ArgConfig, ArgType } from "./types";
/**
 * Cast a string value to the given type.
 *
 * @param value The string value to cast.
 * @param type The type to cast the value to.
 * @param multiple Whether the value is a multiple value.
 * @param defaultValue The default value if the value is empty.
 * @returns The cast value.
 *
 * @category Utils
 */
export declare function castArgValue(value: string, type: ArgType, multiple: boolean, defaultValue?: unknown): unknown;
/**
 * Parse a string arguments to a plain object.
 *
 * @param argsString The string arguments.
 * @returns The parsed plain object.
 *
 * @category Utils
 */
export declare function parseArgsString(argsString: string): Record<string, string>;
/**
 * Generates help content for this argument.
 *
 * @param argConfig - The argument configuration.
 * @returns The help content.
 *
 * @category Utils
 */
export declare function formatArgHelp(argConfig: ArgConfig): [string, string][];
/**
 * Converts a two-dimensional array of strings into a table.
 *
 * @param data The two-dimensional array of strings.
 * @param padding The amount of padding to add to each cell.
 * @returns The table as a string.
 *
 * @category Utils
 */
export declare function convertToTable(data: string[][], padding?: number): string;
