"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.castArgValue = castArgValue;
exports.parseArgsString = parseArgsString;
exports.formatArgHelp = formatArgHelp;
exports.convertToTable = convertToTable;
/**
 * Cast a string value to the given type.
 *
 * @param value The string value to cast.
 * @param type The type to cast the value to.
 * @param multiple Whether the value is a multiple value.
 * @param defaultValue The default value if the value is empty.
 * @returns The cast value.
 *
 * @category Utils
 */
function castArgValue(value, type, multiple, defaultValue) {
    if (multiple) {
        var result = value.split(' ').filter(function (x) { return x !== ''; }).map(function (x) { return castArgValue(x, type, false); });
        return result.length > 0 ? result : (defaultValue !== null && defaultValue !== void 0 ? defaultValue : []);
    }
    var _value = value !== '' ? value : (defaultValue !== null && defaultValue !== void 0 ? defaultValue : value);
    if (_value === undefined) {
        return _value;
    }
    switch (type) {
        case 'string': return _value;
        case 'number': return Number(_value);
        case 'boolean': return _value !== 'false' && _value !== '0';
    }
}
/**
 * Parse a string arguments to a plain object.
 *
 * @param argsString The string arguments.
 * @returns The parsed plain object.
 *
 * @category Utils
 */
function parseArgsString(argsString) {
    return Object.fromEntries(parseArgsArray(splitArgsString(formatArgsString(argsString))));
}
/**
 * Generates help content for this argument.
 *
 * @param argConfig - The argument configuration.
 * @returns The help content.
 *
 * @category Utils
 */
function formatArgHelp(argConfig) {
    var _a;
    var result = [];
    result.push([formatArgUsageExample(argConfig), (_a = argConfig.description) !== null && _a !== void 0 ? _a : '<no description>']);
    result.push(['', "Type: ".concat(formatArgType(argConfig))]);
    if (argConfig.default !== undefined) {
        result.push(['', "Default value: ".concat(JSON.stringify(argConfig.default))]);
    }
    if (argConfig.allowedValues !== undefined) {
        result.push(['', "Allowed values: ".concat(argConfig.allowedValues.join(', '))]);
    }
    return result;
}
/**
 * Converts a two-dimensional array of strings into a table.
 *
 * @param data The two-dimensional array of strings.
 * @param padding The amount of padding to add to each cell.
 * @returns The table as a string.
 *
 * @category Utils
 */
function convertToTable(data, padding) {
    var e_1, _a;
    if (padding === void 0) { padding = 0; }
    if (!data || data.length === 0) {
        return '';
    }
    var maxCols = Math.max.apply(Math, __spreadArray([], __read(data.map(function (row) { return row.length; })), false));
    var normalizedData = data.map(function (row) {
        return row.concat(Array(maxCols - row.length).fill(''));
    } // Добавляем пустые строки где нужно
    );
    var columnWidths = Array(maxCols).fill(0);
    var _loop_1 = function (col) {
        columnWidths[col] = Math.max.apply(Math, __spreadArray([], __read(normalizedData.map(function (row) { var _a; return (((_a = row[col]) === null || _a === void 0 ? void 0 : _a.length) || 0); })), false));
    };
    for (var col = 0; col < maxCols; col++) {
        _loop_1(col);
    }
    var result = [];
    try {
        for (var normalizedData_1 = __values(normalizedData), normalizedData_1_1 = normalizedData_1.next(); !normalizedData_1_1.done; normalizedData_1_1 = normalizedData_1.next()) {
            var row = normalizedData_1_1.value;
            var rowString = row
                .map(function (cell, i) { return cell.padEnd(columnWidths[i] + padding, ' '); }) // Добавляем пробелы до максимальной длины
                .join(' '); // Разделитель
            result.push(rowString);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (normalizedData_1_1 && !normalizedData_1_1.done && (_a = normalizedData_1.return)) _a.call(normalizedData_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return result.join('\n');
}
/**
 * Generates a string representation of the argument type.
 *
 * @param argConfig - The argument configuration.
 * @returns The string representation of the argument type.
 *
 * @category Utils
 */
function formatArgType(argConfig) {
    var result = argConfig.multiple ? "Array<".concat(argConfig.type, ">") : argConfig.type;
    if (argConfig.notEmpty && argConfig.type !== 'boolean') {
        result += ' (not empty)';
    }
    return result;
}
/**
 * Generates an example of the argument usage.
 *
 * @param argConfig - The argument configuration.
 * @returns The example of the argument usage.
 *
 * @category Utils
 */
function formatArgUsageExample(argConfig) {
    var result = '';
    var argValueExample = formatArgValueExample(argConfig);
    if (argConfig.alias) {
        result += "".concat(argConfig.alias).concat(argValueExample, ", ").concat(argConfig.name).concat(argValueExample);
    }
    else {
        result += "".concat(argConfig.name).concat(argValueExample);
    }
    return result;
}
/**
 * Generates an example of the argument value.
 *
 * @param argConfig - The argument configuration.
 * @returns The example of the argument value.
 *
 * @category Utils
 */
function formatArgValueExample(argConfig) {
    if (argConfig.type === 'boolean' && !argConfig.multiple) {
        return '';
    }
    if (argConfig.multiple) {
        return " <".concat(argConfig.type, "> <").concat(argConfig.type, "> ...");
    }
    return " <".concat(argConfig.type, ">");
}
/**
 * Parse an array of string arguments to an array of tuples.
 *
 * @param args The array of string arguments.
 * @returns The parsed array of tuples.
 *
 * @category Utils
 */
function parseArgsArray(args) {
    return args
        .map(function (x) { return x.split(/ (.+)/).slice(0, 2); })
        .map(function (x) { return x.length === 1 ? [x[0], ''] : x; })
        .filter(function (x) { return x[0] !== ''; });
}
/**
 * Split an arguments string to an array of strings.
 *
 * @param argsString The string arguments.
 * @returns The split array of strings.
 *
 * @category Utils
 */
function splitArgsString(argsString) {
    return argsString.split(' -').map(function (x, i) { return i === 0 ? x : "-".concat(x); });
}
/**
 * Format an arguments string.
 *
 * @param argsString The string arguments.
 * @returns The formatted string.
 *
 * @category Utils
 */
function formatArgsString(argsString) {
    return argsString
        .trim()
        .replace(/\s+/g, ' ')
        .replace(/^(-[A-Za-z0-9]{2,})/, function (_, p) { return ' ' + formatGluedArgsString(p); })
        .replace(/ (-[A-Za-z0-9]{2,})/, function (_, p) { return ' ' + formatGluedArgsString(p); })
        .replace(/\s+/g, ' ')
        .trim();
}
/**
 * Format a glued arguments string.
 *
 * @param gluedArgsString The glued arguments string.
 * @returns The formatted string.
 *
 * @category Utils
 */
function formatGluedArgsString(gluedArgsString) {
    return gluedArgsString.slice(1).split('').map(function (x) { return "-".concat(x); }).join(' ');
}
//# sourceMappingURL=utils.js.map